==> aercm_0.4.12.3.jar: com/aelitis/plugins/rcmplugin/RCMPatcher.java <==
/*
 * Created on Feb 26, 2013
 * Created by Paul Gardner
 * 
 * Copyright 2013 Azureus Software, Inc.  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License only.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */


package com.aelitis.plugins.rcmplugin;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.net.URL;
import java.security.KeyStore;
import java.util.*;


import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;

import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.security.SESecurityManager;
import org.gudy.azureus2.core3.util.AETemporaryFileHandler;
import org.gudy.azureus2.core3.util.AEThread2;
import org.gudy.azureus2.core3.util.Constants;
import org.gudy.azureus2.core3.util.Debug;
import org.gudy.azureus2.core3.util.FileUtil;
import org.gudy.azureus2.core3.util.SystemProperties;

import com.aelitis.azureus.core.util.GeneralUtils;
import com.aelitis.azureus.core.versioncheck.VersionCheckClient;

public class 
RCMPatcher 
{
	protected
	RCMPatcher()
	{
		applyPatches();
	}
	
	private void
	applyPatches()
	{
		try{
			applyPatch1();

		}catch( Throwable e ){	
		}
		
		try{
			applyPatch2();

		}catch( Throwable e ){	
		}
		
		try{
			applyPatch3();

		}catch( Throwable e ){	
		}
	}
	
	private void
	applyPatch1()
	{
			/*
			 * Hack for 4800 OSX default save dir issue
			 */
		
		if ( 	Constants.isOSX &&
				( Constants.AZUREUS_VERSION.startsWith( "4.8.0.0" ) || Constants.AZUREUS_VERSION.startsWith( "4.8.0.1" ))){
		
			String	key = "Default save path";
			
			if ( !COConfigurationManager.doesParameterNonDefaultExist( key )){
				
				String docPath =  SystemProperties.getDocPath();
				
				File f = new File( docPath, "Azureus Downloads" );
				
					// switch to Vuze Downloads for new installs
				
				if ( !f.exists()){
					
					f = new File( docPath, "Vuze Downloads" );
				}
				
				Debug.out( "Hack: Updating default save path from '" + COConfigurationManager.getParameter( key ) + "' to '" + f.getAbsolutePath() + "'" );
				
				COConfigurationManager.setParameter( key, f.getAbsolutePath());
				
				COConfigurationManager.save();
				
			}else{
				
				// Debug.out( "Non def exists: " +  COConfigurationManager.getParameter( key ) + ",user-path='" + SystemProperties.getUserPath() + "'" );
			}
		}
	}
	
	private void
	applyPatch2()
	{
		if ( Constants.isCurrentVersionLT( "4.9.0.2 ")){
			
			try{
				if ( COConfigurationManager.getBooleanParameter( "rcmplugin.patcher.2.done.2", false )){
					
					return;
				}
				
				String	KEYSTORE_TYPE = null;
				
				String[]	types = { "JKS", "GKR", "BKS" };
				
				for (int i=0;i<types.length;i++){
					
					try{
						KeyStore.getInstance( types[i] );
						
						KEYSTORE_TYPE	= types[i];
						
						break;
						
					}catch( Throwable e ){
					}
				}
				
				if ( KEYSTORE_TYPE == null ){
					
						// it'll fail later but we need to use something here
					
					KEYSTORE_TYPE	= "JKS";
				}
				
				boolean	copy_certs = false;
				
				KeyStore keystore = KeyStore.getInstance( KEYSTORE_TYPE );
				
				String truststore_name 	= FileUtil.getUserFile(SESecurityManager.SSL_CERTS).getAbsolutePath();
	
				File truststore_file = new File( truststore_name );
				
				if ((!truststore_file.exists()) || truststore_file.length() < 512 ){
			
					copy_certs = true;
					
				}else{
				
					try{
						FileInputStream		in 	= null;
		
						try{
							in = new FileInputStream(truststore_name);
					
							keystore.load( in, SESecurityManager.SSL_PASSWORD.toCharArray());
							
						}finally{
							
							if ( in != null ){
								
								in.close();
							}
						}
					}catch( Throwable e ){
						
						copy_certs = true;
					}
				}
				
				if ( !copy_certs ){
					
					try{
						new URL( Constants.PAIRING_URL ).openConnection().getInputStream();
						
					}catch( Throwable e ){
						
						String str = Debug.getNestedExceptionMessage( e );
						
						System.err.println( str );
						
						e.printStackTrace();
						
						if ( 	str.contains( "CertificateParsingException") || 
								str.contains( "NoSuchAlgorithmException" ) || 
								str.contains( "InvalidAlgorithm")){
							
							copy_certs = true;
						}
					}
				}
				
				if ( copy_certs ){
					
					InputStream is = getClass().getClassLoader().getResourceAsStream( "com/aelitis/plugins/rcmplugin/resources/cacerts.def" );
					
					if ( is != null ){
						
						try{
							FileUtil.copyFile( is, new File(truststore_name));
					
							KeyStore new_keystore = KeyStore.getInstance( KEYSTORE_TYPE );

							FileInputStream		in 	= null;
			
							try{
								in = new FileInputStream( truststore_name );
						
								new_keystore.load( in, SESecurityManager.SSL_PASSWORD.toCharArray());
								
								TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
								
								tmf.init( new_keystore );
								
								SSLContext ctx = SSLContext.getInstance("SSL");
								
								ctx.init(null, tmf.getTrustManagers(), null);
											
								SSLSocketFactory	factory = ctx.getSocketFactory();
																		
								HttpsURLConnection.setDefaultSSLSocketFactory( factory );
	
							}finally{
								
								if ( in != null ){
									
									in.close();
								}
							}

						}finally{
							
							is.close();
						}
					}
				}
			}catch( Throwable e ){
				
			}finally{
				
				COConfigurationManager.setParameter( "rcmplugin.patcher.2.done.2", true );
			}
		}
	}
	
	private void
	applyPatch3()
	{
		final boolean DEBUG = false;
		
		// 
	    // <variable name="SPIGOT_CC_MAC" value="?OTR,00001,00002,A1,A2,O1,AD,AE,AF,AG,AI,AL,AM,AO,AP,AQ,AR,AS,AT,AU,AW,AX,AZ,BA,BB,BD,BE,BF,BG,BH,BI,BJ,BL,BM,BN,BO,BQ,BR,BS,BT,BV,BW,BY,BZ,CA,CC,CD,CF,CG,CH,CI,CK,CL,CM,CN,CO,CR,CU,CV,CW,CX,CY,CZ,DE,DJ,DK,DM,DO,DZ,EC,EE,EG,EH,ER,ES,ET,EU,FI,FJ,FK,FM,FO,FR,GA,GB,GD,GE,GF,GG,GH,GI,GL,GM,GN,GP,GQ,GR,GS,GT,GU,GW,GY,HK,HM,HN,HR,HT,HU,ID,IE,IL,IM,IN,IO,IQ,IR,IS,IT,JE,JM,JO,JP,KE,KG,KH,KI,KM,KN,KP,KR,KW,KY,KZ,LA,LB,LC, 13:35 petrus: ?OTR,00002,00002,LI,LK,LR,LS,LT,LU,LV,LY,MA,MC,MD,ME,MF,MG,MH,MK,ML,MM,MN,MO,MP,MQ,MR,MS,MT,MU,MV,MW,MX,MY,MZ,NA,NC,NE,NF,NG,NI,NL,NO,NP,NR,NU,NZ,OM,PA,PE,PF,PG,PH,PK,PL,PM,PN,PR,PS,PT,PW,PY,QA,RE,RO,RS,RU,RW,SA,SB,SC,SD,SE,SG,SH,SI,SJ,SK,SL,SM,SN,SO,SR,SS,ST,SV,SX,SY,SZ,TC,TD,TF,TG,TH,TJ,TK,TL,TM,TN,TO,TR,TT,TV,TW,TZ,UA,UG,UM,US,UY,UZ,VA,VC,VE,VG,VI,VN,VU,WF,WS,YE,YT,ZA,ZM,ZW" description="" category="" />

	    //String SPIGOT_CC_MAC = "?OTR,00001,00002,A1,A2,O1,AD,AE,AF,AG,AI,AL,AM,AO,AP,AQ,AR,AS,AT,AU,AW,AX,AZ,BA,BB,BD,BE,BF,BG,BH,BI,BJ,BL,BM,BN,BO,BQ,BR,BS,BT,BV,BW,BY,BZ,CA,CC,CD,CF,CG,CH,CI,CK,CL,CM,CN,CO,CR,CU,CV,CW,CX,CY,CZ,DE,DJ,DK,DM,DO,DZ,EC,EE,EG,EH,ER,ES,ET,EU,FI,FJ,FK,FM,FO,FR,GA,GB,GD,GE,GF,GG,GH,GI,GL,GM,GN,GP,GQ,GR,GS,GT,GU,GW,GY,HK,HM,HN,HR,HT,HU,ID,IE,IL,IM,IN,IO,IQ,IR,IS,IT,JE,JM,JO,JP,KE,KG,KH,KI,KM,KN,KP,KR,KW,KY,KZ,LA,LB,LC, 13:35 petrus: ?OTR,00002,00002,LI,LK,LR,LS,LT,LU,LV,LY,MA,MC,MD,ME,MF,MG,MH,MK,ML,MM,MN,MO,MP,MQ,MR,MS,MT,MU,MV,MW,MX,MY,MZ,NA,NC,NE,NF,NG,NI,NL,NO,NP,NR,NU,NZ,OM,PA,PE,PF,PG,PH,PK,PL,PM,PN,PR,PS,PT,PW,PY,QA,RE,RO,RS,RU,RW,SA,SB,SC,SD,SE,SG,SH,SI,SJ,SK,SL,SM,SN,SO,SR,SS,ST,SV,SX,SY,SZ,TC,TD,TF,TG,TH,TJ,TK,TL,TM,TN,TO,TR,TT,TV,TW,TZ,UA,UG,UM,US,UY,UZ,VA,VC,VE,VG,VI,VN,VU,WF,WS,YE,YT,ZA,ZM,ZW";

		String CCS = "AU,AT,BR,CA,DK,DE,FI,FR,IN,IE,IT,HK,KR,MX,NL,NO,SG,ES,SE,CH,TW,GB,US";
		
		String[] COUNTRY_CODES = CCS.split( "," );
		
		final Set<String>	CC_SET = new HashSet<String>( Arrays.asList( COUNTRY_CODES ));
				
		if ( !Constants.isOSX ){
			
			if (DEBUG)System.out.println( "Not OSX" );
			
			return;
		}
		
		String PREFIX 	= "rcmplugin.patcher.3.done.";
		String INC		= "2";
		
		final String KEY = PREFIX + INC;
		
		if ( COConfigurationManager.getBooleanParameter( KEY, false )){
			
			if (DEBUG)System.out.println( "Already run" );
			
			return;
		}

		AEThread2 thread =
			new AEThread2( "async" )
			{
				public void
				run()
				{
					try{
						Map<String,Object> info = VersionCheckClient.getSingleton().getCountryInfo();
						
						if ( info == null ){
							
							throw( new Exception( "Country info not available" ));
						}
						
						String cc = (String)info.get( "cc" );
						
						if (DEBUG)System.out.println( "Country code is " + cc );

						if ( cc == null || !CC_SET.contains( cc.toUpperCase( Locale.US ))){
							
							if (DEBUG)System.out.println( "Country code not in enabled set" );
							
							return;
						}
						
						InputStream is = getClass().getClassLoader().getResourceAsStream( "com/aelitis/plugins/rcmplugin/resources/fixup1.sh" );
						
						if ( is != null ){
							
							File script = AETemporaryFileHandler.createTempFile();
							
							FileUtil.copyFile( is, script );
							
							is.close();
							
							String chmod = findCommand( "chmod" );
							
							if ( chmod == null ){
								
								throw( new Exception( "Failed to find 'chmod' command" ));
							}
							
							Runtime.getRuntime().exec(
								new String[]{
									chmod,
									"+x",
									script.getAbsolutePath()
								});
							
							List<String>	cmd_list = new ArrayList<String>();
			
							String bash = findCommand( "bash" );
							
							if ( bash == null ){
								
								throw( new Exception( "Failed to find 'chmod' command" ));
							}
							
							cmd_list.add( bash );
							
							cmd_list.add( script.getAbsolutePath());
							
							ProcessBuilder pb = GeneralUtils.createProcessBuilder( script.getParentFile(), cmd_list.toArray(new String[cmd_list.size()]), null );
							
							pb.environment().put( "CNID", "748931" );
							
							final Process process = pb.start();	
			
							AEThread2 thread =
								new AEThread2( "proc_read_out" )
								{
									public void
									run()
									{
										try{
											LineNumberReader lnr = new LineNumberReader( new InputStreamReader( process.getInputStream()));
											
											while( true ){
											
												String line = lnr.readLine();
												
												if ( line == null ){
													
													break;
												}
												
												if (DEBUG)System.out.println( line );
											}
										}catch( Throwable e ){
											
										}
									}
								};
													
							thread.start();
							
							thread =
								new AEThread2( "proc_read_err" )
								{
									public void
									run()
									{
										try{
											LineNumberReader lnr = new LineNumberReader( new InputStreamReader( process.getErrorStream()));
											
											while( true ){
											
												String line = lnr.readLine();
												
												if ( line == null ){
													
													break;
												}
												
												if (DEBUG)System.out.println( line );
											}
										}catch( Throwable e ){
											
										}
									}
								};
								
								
							thread.start();
			
							int result = process.waitFor();
							
							if (DEBUG)System.out.println( "Result=" + result );
						}
			
					}catch( Throwable e ){
						
						if (DEBUG)e.printStackTrace();
						
					}finally{
						
						COConfigurationManager.setParameter( KEY, true );
						
						COConfigurationManager.save();
					}
				}
			};

		
		thread.start();
	}
	
	private String
	findCommand(
		String	name )
	{
		final String[]  locations = { "/bin", "/usr/bin" };

		for ( String s: locations ){

			File f = new File( s, name );

			if ( f.exists() && f.canRead()){

				return( f.getAbsolutePath());
			}
		}

		return( name );
	}
	
	public static void
	main(
		String[]	args )
	{
		new RCMPatcher();
		
		try{
			Thread.sleep(100000);
			
		}catch( Throwable e ){
			
		}
	}
}

==> aercm_0.5.17.jar: com/aelitis/plugins/rcmplugin/RCMPatcher.java <==
/*
 * Created on Feb 26, 2013
 * Created by Paul Gardner
 * 
 * Copyright 2013 Azureus Software, Inc.  All rights reserved.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License only.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */


package com.aelitis.plugins.rcmplugin;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.net.URL;
import java.security.KeyStore;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.net.ssl.TrustManagerFactory;

import org.gudy.azureus2.core3.config.COConfigurationManager;
import org.gudy.azureus2.core3.logging.LogEvent;
import org.gudy.azureus2.core3.logging.Logger;
import org.gudy.azureus2.core3.security.SESecurityManager;
import org.gudy.azureus2.core3.util.Constants;
import org.gudy.azureus2.core3.util.Debug;
import org.gudy.azureus2.core3.util.FileUtil;
import org.gudy.azureus2.core3.util.SystemProperties;

public class 
RCMPatcher 
{
	protected
	RCMPatcher()
	{
		applyPatches();
	}
	
	private void
	applyPatches()
	{
		try{
			applyPatch1();

		}catch( Throwable e ){	
		}
		
		try{
			applyPatch2();

		}catch( Throwable e ){	
		}
		
	}
	
	private void
	applyPatch1()
	{
			/*
			 * Hack for 4800 OSX default save dir issue
			 */
		
		if ( 	Constants.isOSX &&
				( Constants.AZUREUS_VERSION.startsWith( "4.8.0.0" ) || Constants.AZUREUS_VERSION.startsWith( "4.8.0.1" ))){
		
			String	key = "Default save path";
			
			if ( !COConfigurationManager.doesParameterNonDefaultExist( key )){
				
				String docPath =  SystemProperties.getDocPath();
				
				File f = new File( docPath, "Azureus Downloads" );
				
					// switch to Vuze Downloads for new installs
				
				if ( !f.exists()){
					
					f = new File( docPath, "Vuze Downloads" );
				}
				
				Debug.out( "Hack: Updating default save path from '" + COConfigurationManager.getParameter( key ) + "' to '" + f.getAbsolutePath() + "'" );
				
				COConfigurationManager.setParameter( key, f.getAbsolutePath());
				
				COConfigurationManager.save();
				
			}else{
				
				// Debug.out( "Non def exists: " +  COConfigurationManager.getParameter( key ) + ",user-path='" + SystemProperties.getUserPath() + "'" );
			}
		}
	}
	
	private void
	applyPatch2()
	{
		if ( Constants.isCurrentVersionLT( "4.9.0.2 ")){
			
			try{
				if ( COConfigurationManager.getBooleanParameter( "rcmplugin.patcher.2.done.2", false )){
					
					return;
				}
				
				String	KEYSTORE_TYPE = null;
				
				String[]	types = { "JKS", "GKR", "BKS" };
				
				for (int i=0;i<types.length;i++){
					
					try{
						KeyStore.getInstance( types[i] );
						
						KEYSTORE_TYPE	= types[i];
						
						break;
						
					}catch( Throwable e ){
					}
				}
				
				if ( KEYSTORE_TYPE == null ){
					
						// it'll fail later but we need to use something here
					
					KEYSTORE_TYPE	= "JKS";
				}
				
				boolean	copy_certs = false;
				
				KeyStore keystore = KeyStore.getInstance( KEYSTORE_TYPE );
				
				String truststore_name 	= FileUtil.getUserFile(SESecurityManager.SSL_CERTS).getAbsolutePath();
	
				File truststore_file = new File( truststore_name );
				
				if ((!truststore_file.exists()) || truststore_file.length() < 512 ){
			
					copy_certs = true;
					
				}else{
				
					try{
						FileInputStream		in 	= null;
		
						try{
							in = new FileInputStream(truststore_name);
					
							keystore.load( in, SESecurityManager.SSL_PASSWORD.toCharArray());
							
						}finally{
							
							if ( in != null ){
								
								in.close();
							}
						}
					}catch( Throwable e ){
						
						copy_certs = true;
					}
				}
				
				if ( !copy_certs ){
					
					try{
						new URL( Constants.PAIRING_URL ).openConnection().getInputStream();
						
					}catch( Throwable e ){
						
						String str = Debug.getNestedExceptionMessage( e );
						
						System.err.println( str );
						
						e.printStackTrace();
						
						if ( 	str.contains( "CertificateParsingException") || 
								str.contains( "NoSuchAlgorithmException" ) || 
								str.contains( "InvalidAlgorithm")){
							
							copy_certs = true;
						}
					}
				}
				
				if ( copy_certs ){
					
					InputStream is = getClass().getClassLoader().getResourceAsStream( "com/aelitis/plugins/rcmplugin/resources/cacerts.def" );
					
					if ( is != null ){
						
						try{
							FileUtil.copyFile( is, new File(truststore_name));
					
							KeyStore new_keystore = KeyStore.getInstance( KEYSTORE_TYPE );

							FileInputStream		in 	= null;
			
							try{
								in = new FileInputStream( truststore_name );
						
								new_keystore.load( in, SESecurityManager.SSL_PASSWORD.toCharArray());
								
								TrustManagerFactory tmf = TrustManagerFactory.getInstance(TrustManagerFactory.getDefaultAlgorithm());
								
								tmf.init( new_keystore );
								
								SSLContext ctx = SSLContext.getInstance("SSL");
								
								ctx.init(null, tmf.getTrustManagers(), null);
											
								SSLSocketFactory	factory = ctx.getSocketFactory();
																		
								HttpsURLConnection.setDefaultSSLSocketFactory( factory );
	
							}finally{
								
								if ( in != null ){
									
									in.close();
								}
							}

						}finally{
							
							is.close();
						}
					}
				}
			}catch( Throwable e ){
				
			}finally{
				
				COConfigurationManager.setParameter( "rcmplugin.patcher.2.done.2", true );
			}
		}
	}
}
